#include <stdio.h>
#include <tchar.h>
#include <string>
#include <sstream>

// There are a few bit hacks that you can find to speed this up. For the sake of simplicity 
// and clarity I've made it with a simple loop
unsigned char findLeftmostBitSet( unsigned int bit )
{
    unsigned char counter = 31;
    while( (bit & 0x80000000) == 0 && counter > 0 )
    {
        counter --;
        bit <<= 1;
    }
    return counter;
}

class Float32
{
public:
    // This constructor creates a Float32 given its string version
    // For example Float32("-21.335"); would represent the number -21.335
    Float32(const std::string& input)
    {
        bool negative;
        unsigned int intPart;
        unsigned int decPart;
        unsigned long long numberOfDecimalDigits;
        convertStringToParts(input, &negative, &intPart, &decPart, &numberOfDecimalDigits);
        convertToFloat(negative, intPart, decPart, numberOfDecimalDigits);
    }

    // Convert to a float
    operator float() const
    {
        unsigned int result = m_negative ? 0x80000000 : 0; // Set the first bit to 1 if negative
        result |= (unsigned int) m_exponent << 23; // Write the exponent in the right position
        result |= m_mantissa >> 9; // Add the mantissa; the right shift is to remove the 9 bits from the 32 of the unsigned int that we have used to store the mantissa
        return *(float*)(&result);
    }

    // To string
    const std::string toString() const
    {
        std::stringstream output;
        output << "Value:: ";
        output << (float)(*this);
        output << "\n";

        output << "Raw:: ";
        output << "(Sign: " << (m_negative ? "- " : "+ ") << ") ";
        output << "(Exp: " << (unsigned int) m_exponent << ") ";
        output << "(Mantissa: 0x" << std::hex << (unsigned int) (m_mantissa & 0x7FFFFF) << ") ";
        output << "\n" << std::dec;

        output << "Binary:: ";
        output << "[" << (m_negative ? "1" : "0") << " ";
        unsigned char counter = 8;
        while( counter > 0 )
        {
            output << ((((m_exponent >> (counter - 1)) & 0x1) == 1) ? "1" : "0");
            counter --;
        }
        output << " ";
        counter = 23;
        while( counter > 0 )
        {
            output << ((((m_mantissa >> (counter + 8)) & 0x1) == 1) ? "1" : "0");
            counter --;
        }
        output << "]";

        //output = m_negative ? "Negative" : "Positive" + std::string("\n");
        return output.str();
    }

private:
    // Convert integer parts to float
    void convertToFloat(bool negative, unsigned int intPart, unsigned int decPart, unsigned long long numberOfDecimalDigits)
    {
        unsigned int intPartBinary;
        unsigned int decPartBinary;
        convertToBinaryReal(intPart, decPart, numberOfDecimalDigits, &intPartBinary, &decPartBinary);
        convertBinaryRealToFloat(intPartBinary, decPartBinary, &m_exponent, &m_mantissa);
        m_negative = negative;
    }

    // Convert a string input into separate components
    // E.g: "-21.0335" would return negativeOut = true, intPartOut = 21, decpartOut = 335, numberOfDecimalDigits = 4
    void convertStringToParts(const std::string& input, bool* negativeOut, unsigned int *intPartOut, unsigned int *decPartOut, unsigned long long *numberOfDecimalDigits) const
    {
        *negativeOut = false;
        *intPartOut = 0;
        *decPartOut = 0;
        * numberOfDecimalDigits = 0;

        if(input.length() > 0)
        {
            std::string temp = input;
            if( temp[0] == '-' )
            {
                *negativeOut = true;
                temp[0] = '0';
            }

            std::size_t found = temp.find_first_of('.');
            if(found != std::string::npos)
            {
                std::string intPartString = temp.substr(0,found);
                *intPartOut = atoi(intPartString.c_str());

                std::string decPartString = temp.substr(found + 1);
                *decPartOut = atoi(decPartString.c_str());
                *numberOfDecimalDigits = decPartString.length();
            }
            else
            {
                *intPartOut = atoi(temp.c_str());
            }
        }
    }

    // Convert integer parts into binary real parts
    // E.g: "5.5" would return intPartBinaryOut = 101b = 5, decPartBinaryOut = 10000000000000000000000000000000b = 2147483648
    void convertToBinaryReal(unsigned int intPart, unsigned int decPart, unsigned long long numberOfDecimalDigits, unsigned int *intPartBinaryOut, unsigned int *decPartBinaryOut) const
    {
        // The integer part is already in a form we like; no conversion is actually needed, just copy it across
        *intPartBinaryOut = intPart;
        // The decimal part needs to be converted
        *decPartBinaryOut = 0;
        long long overflowContainer = decPart;
        long long overflowMask = 1;
        for(unsigned long long i = 0; i < numberOfDecimalDigits; ++i) overflowMask *= 10;
        // Run through all the output bits we have, and for each one calculate its state
        for(unsigned int bitIndex = 1; bitIndex <= sizeof(unsigned int) * 8 && overflowContainer != 0; ++bitIndex)
        {
            overflowContainer *= 2;
            if(overflowContainer - overflowMask >= 0)
            {
                *decPartBinaryOut |= 1 << (sizeof(unsigned int) * 8 - bitIndex);
                overflowContainer -= overflowMask;
            }
        }
    }

    // Convert binary real parts into exponent and mantissa
    // E.g: intPartBinary = 101b = 5, decPartBinary = 10000000000000000000000000000000b = 2147483648 generate an output of *exponentOut = 129 and *mantissaOut = 1610612736
    void convertBinaryRealToFloat(unsigned int intPartBinary, unsigned int decPartBinary, unsigned char *exponentOut, unsigned int *mantissaOut ) const
    {
        int uncompressedExponent = 0;
        // If the integer part is 0 we will get a negative exponent
        if(intPartBinary == 0)
        {
            // 0.0b is a special case
            if(decPartBinary == 0)
            {
                uncompressedExponent = -127;
                *mantissaOut = 0;
            }
            else
            {
                uncompressedExponent = 32 - findLeftmostBitSet(decPartBinary);
                *mantissaOut = decPartBinary << uncompressedExponent;
                uncompressedExponent = -uncompressedExponent;
            }
        }
        // else we get a positive exponent
        else
        {
            uncompressedExponent = findLeftmostBitSet(intPartBinary);
            *mantissaOut = decPartBinary >> uncompressedExponent;
            *mantissaOut = *mantissaOut | (intPartBinary << (32 - uncompressedExponent));
        }
        *exponentOut = uncompressedExponent + 127;
    }

    // Fields
    bool            m_negative;
    unsigned char   m_exponent;
    unsigned int    m_mantissa;
};

int _tmain(int argc, _TCHAR* argv[])
{
    Float32 float32("-123.321");
    printf("%s",float32.toString().c_str());
	return 0;
}

